# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - PostMessage Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinMoin.PageEditor import PageEditor
from MoinSupport import getMetadata, writeHeaders
from MoinMessage import is_collection, to_replace, to_store
from MoinMessageSupport import MoinMessageAction

Dependencies = ['pages']

class PostMessage(MoinMessageAction):

    "A posted message handler."

    def handle_message_object(self, message):

        "Handle the given 'message' object."

        request = self.request

        # Handle each update.

        for update in message.updates:

            # Handle a single part.

            if not is_collection(update):
                self.handle_message_parts([update], update)

            # Or a collection of alternative representations for a single
            # update.

            else:
                self.handle_message_parts(update.get_payload(), update)

        # Default output.

        writeHeaders(request, "text/plain", getMetadata(self.page), "204 No Content")

    def handle_message_parts(self, parts, update):

        """
        Handle the given message 'parts', using the original 'update' to
        determine whether the content is to replace or update page content, or
        whether it will be placed in a message store.
        """

        request = self.request

        # Handle the different update actions.
        # Update a message store for the page.

        if to_store(update):

            # Add any authenticated user.
            # Note that where messages are signed by the real author, encrypted,
            # and then signed for sending, the authenticated user here is not
            # the real author.

            update["Moin-User"] = request.user and request.user.valid and request.user.name or None

            self.store.append(update.as_string())

        # Update the page.

        else:
            # NOTE: Should either choose preferred content types or somehow retain them
            # NOTE: all but present one at a time.

            body = []
            replace = to_replace(update)

            for part in parts:
                mimetype = part.get_content_type()
                encoding = part.get_content_charset()
                if mimetype == "text/moin":
                    payload = part.get_payload(decode=True)
                    body.append(encoding and unicode(payload, encoding) or payload)
                    if replace:
                        break

            if not replace:
                body.append(self.page.get_raw_body())

            page_editor = PageEditor(request, self.pagename)
            page_editor.saveText("\n\n".join(body), 0)

            # Refresh the page.

            self.page = Page(request, self.pagename)

# Action function.

def execute(pagename, request):
    PostMessage(pagename, request).do_action() # instead of render

# vim: tabstop=4 expandtab shiftwidth=4
